/*******************************************************************************/
/* This program is for demonstrating how institution and branch data can be    */ 
/* loaded for FinCEN regulatory reports.                                       */
/* The data should be adjusted to match with your organizational information.  */
/*******************************************************************************/
%macro rc_load_sample_location; 

/*******************************************************************************/
/* Sample transmitter data.                                                    */ 
/* Shared by all finCEN forms.                                                 */
/* There should be only one current record for each tax ID.                    */
/* The effective period should not have gaps or overlaps.                      */
/* Transmitter_key should be unique.                                           */
/*******************************************************************************/

data fincen_transmitter;
    if 0 then set rc_cfg.fincen_transmitter;
    transmitter_key=1;
    transmitter_name='SAS Institute Inc.';                                                                                                    
    transmitter_address='100 SAS Campus Drive';                                                                                           
    transmitter_city_name='Cary';                                                                                               
    transmitter_state_code='NC';                                                                                              
    transmitter_zip_code='27513';                                                                                                
    transmitter_country_code='US' ;
    transmitter_contact_name='SAS TECHNICAL SUPPORT'; 
    transmitter_telephone='9196778008';                                                                                                                                                                                          
    transmitter_tax_id='123123123';                                                                                                
    transmitter_control_code='TBSATEST'; 
    effective_from_date=mdy(1,1,2000)*24*3600;
    effective_to_date=mdy(1,1,5999)*24*3600;
    current_ind='Y'; 
    output;
run; 




%if &rc_debug = N %then
%do;
    proc sql;

         delete from rc_cfg.fincen_transmitter;
         
    quit;
    
    proc append base=rc_cfg.fincen_transmitter data=fincen_transmitter  force;
    run;
     
%end;


/*********************************************************************************/
/* Sample institution data.                                                      */
/* This example shows that the institution information is the same for all forms */
/* but the contact information can be different.                                 */
/*                                                                               */ 
/* There should be only one current record for each form.                        */
/* The effective period should not have gaps or overlaps.                        */
/* Institution_key should be unique.                                             */
/*********************************************************************************/

data fincen_institution;

    if 0 then set rc_cfg.fincen_institution;
    
    institution_key=1;
    institution_legal_name='SAS Institute Inc.'; 
    institution_alternate_name='SAS';  
    institution_address='100 SAS Campus Drive';                                                                                           
    institution_city_name='Cary';                                                                                               
    institution_state_code='NC';                                                                                              
    institution_zip_code='27513';                                                                                                
    institution_country_code='US' ;                                                                                                                                                                                     
    institution_tax_id='123123123'; 
    institution_prim_fed_reg_code='C'; 
    institution_tax_id_type='A';
    institution_type_code='E';
    institution_id_type_code='D';
    institution_id='111222333';
    institution_sf_type_code='DF';
    institution_alternate_name='';
    institution_gaming_type_code = 'Z';
    institution_gaming_type_other='Other gaming';
    /* contact information */ 
    contact_office='COMPLIANCE';
    contact_name='';
    contact_title='COMPLIANCE OFFICER';
    contact_phone_number='9196778008';
    contact_phone_extension='';
    current_ind='Y'; 
    effective_from_date=mdy(1,1,2000)*24*3600;
    effective_to_date=mdy(1,1,5999)*24*3600;
    form_config_key=301;
    output;
    institution_key=2;
    form_config_key=302;
    contact_office='COMPLIANCE';
    contact_name='';
    contact_title='';
    contact_phone_number='9196778008';
    contact_phone_extension='';
    output;
    institution_key=3;
    form_config_key=303;
    contact_office='COMPLIANCE';
    contact_name='OFFICE A';
    contact_title='COMPLIANCE OFFICER';
    contact_phone_number='9196778008';
    contact_phone_extension='';
    output;

run;



%if &rc_debug = N %then
%do;
    proc sql;

         delete from rc_cfg.fincen_institution;
         
    quit;
    
    proc append base=rc_cfg.fincen_institution data=fincen_institution force;
    run;
     
%end;



/**********************************************************************************/
/* Sample branch data.                                                            */
/* This example shows how to integrate FSC_BRANCH_DIM data with branch            */
/* information in .csv that is specific for FinCEN e-files.                       */
/*                                                                                */
/* Branch number is used to merge data.                                           */
/* Same list of branches for SAR and CTR but different fields are used for        */
/* different forms.                                                               */
/**********************************************************************************/

data branch_extra;
      
     if 0 then set rc_cfg.fincen_branch;

     length effective_from_date_str 
            effective_to_date_str $ 20
            ;

     infile  rc_tmplt(rc_fincen_sample_branch_data.csv)  
             dsd lrecl=32764 truncover firstobs=2 delimiter=',' ;
     input 
          BRANCH_NUMBER             $ 
          BRANCH_ID                 $ 
          BRANCH_RSSD_ID            $
          BRANCH_GAMING_TYPE_CODE   $
          BRANCH_GAMING_TYPE_OTHER  $
          BRANCH_ID_TYPE_CODE       $
          BRANCH_PRIM_FED_REG_CODE  $
          BRANCH_TAX_ID             $
          BRANCH_TYPE_CTR_CODE      $
          BRANCH_TYPE_CTR_OTHER     $       
          effective_from_date_str $ 
          effective_to_date_str $
           
    ;

    
    effective_from_date=input(substr(effective_from_date_str,1,9),date9.)*24*3600;
    effective_to_date=input(substr(effective_to_date_str,1,9),date9.)*24*3600;

    keep branch_number branch_id branch_rssd_id branch_prim_fed_reg_code branch_tax_id
         branch_id_type_code branch_type_ctr_code branch_type_ctr_other branch_gaming_type_code
         branch_gaming_type_other effective_from_date effective_to_date
         ;

run;

proc sql;
    create table branch_combined as
    select extra.*, 
          dim.branch_key, 
          dim.branch_name length=100,
          dim.street_address_1,
          dim.street_address_2,
          dim.street_city_name,
          dim.street_state_code,
          dim.street_postal_code 
     from branch_extra as extra,
          seg_kc.fsc_branch_dim as dim
     where extra.branch_number=dim.branch_number and
            dim.branch_key > 0
     ;
quit;

data fincen_branch;
     if 0 then set rc_cfg.fincen_branch;
     set branch_combined; 

     branch_country_code='US';
      
     if street_address_1 ne '$' then
     do;
         if street_address_2 ne '$' then branch_address=strip(street_address_1)||' '||strip(street_address_2);
         else branch_address=street_address_1;
     end;
     if street_city_name ne '$' then branch_city_name=street_city_name;
     else branch_city_name='';
     if street_state_code ne '$' then branch_state_code=street_state_code;
     else branch_state_code='';
     if street_postal_code ne '$' then branch_zip_code=street_postal_code;
     else branch_zip_code='';
     drop 
          street_address_1 street_address_2  street_city_name street_state_code
                                     street_postal_code 
    ;
run;



%if &rc_debug = N %then
%do;
    proc sql;

         delete from rc_cfg.fincen_branch;
         
    quit;
    
   proc append base=rc_cfg.fincen_branch data=fincen_branch  force;
   run;
     
%end;

 

%mend;

%macro rc_load_sample_default (form_config_key=); 
   /*******************************************************/
   /* Load form specific information                      */
   /*******************************************************/
 
    
    /* Define default values in RC_CFG.FORM_DEFAULT        */
    /* Only TRANSMITTER_TAX_ID is used at this point.      */
    /* This example is using the same tax ID for all forms */

    data form_default_&form_config_key;
        if 0 then set rc_cfg.form_default;
        retain form_config_key &form_config_key;
        default_attr_name='TRANSMITTER_TAX_ID';
        default_char_value='123123123';
        output;
        
    run;

    %if &rc_debug = N %then
    %do;
        proc sql;
             delete from rc_cfg.form_default
             where form_config_key=&form_config_key ;
        quit;
        proc append base=rc_cfg.form_default data=form_default_&form_config_key force;
        run;
    %end;

     
    /* load institution data into json for UI display */
    /* The data is defined in the institution should only be one current institution.  */ 

 
    %rc_efile_wrt_entity_info(indsn=rc_cfg.fincen_institution,
                              entity_type_name=FINCEN_INSTITUTION_%trim(&form_config_key),
                              entity_level_code=INSTITUTION,
                              entity_key_var=institution_key,
                              entity_number_var=institution_id,
                              entity_name_var=institution_legal_name,
                              entity_description_var=%nrbquote(cat(strip(institution_address),strip(institution_city_name))), 
                              entity_from_date_var=effective_from_date,
                              entity_to_date_var=effective_to_date,
                              where_cl=%str(form_config_key=&form_config_key)
                                        );
%mend;



%rc_load_sample_location; 

%rc_load_sample_default(form_config_key=301); 
%rc_load_sample_default(form_config_key=302);
%rc_load_sample_default(form_config_key=303); 
 


/* load branch data into json for UI display */
%rc_efile_wrt_entity_info(indsn=rc_cfg.fincen_branch,
                          entity_type_name=FINCEN_BRANCH,
                          entity_level_code=BRANCH,
                          entity_key_var=branch_key,
                          entity_number_var=branch_number,
                          entity_name_var=branch_name,
                          entity_description_var=%nrbquote(cat(strip(branch_address),strip(branch_city_name))), 
                          entity_from_date_var=effective_from_date,
                          entity_to_date_var=effective_to_date
                          );





%rc_efile_wrt_entity_info(indsn=rc_cfg.fincen_transmitter,
                          entity_type_name=FINCEN_TRANSMITTER,
                          entity_level_code=TRANSMITTER,
                          entity_key_var=transmitter_key,
                          entity_number_var=transmitter_tax_id,
                          entity_name_var=transmitter_name,
                          entity_description_var=transmitter_control_code, 
                          entity_from_date_var=effective_from_date,
                          entity_to_date_var=effective_to_date,
                          where_cl=
                          );


