/*==============================================================================
|                               SAS SAMPLE FILE
|
| NAME:    cdd_entity_risk_prep.sas
| PRODUCT: SAS Customer Due Diligence
| SYSTEM:  UNIX/WINDOWS
================================================================================


PREP FILE DESCRIPTION
---------------------
Creates entity_risk_prep data set


PREP FILE NOTES
---------------
None


PREP DATA SET(S) CREATED
------------------------
CM_PREP.CDDDS_ENTITY_RISK_PREP


PREP DATA SET(S) SORT ORDER
----------------------------

| Data Set                         Field                           Sort Order
|--------------------------------|------------------------------ -----------------|--------------
|CDDDS_ENTITY_RISK_PREP          |PRIMARY_ENTITY_NUMBER (AKA PARTY_NUMBER)        |1
|                                |ATTRIBUTE_CODE                                  |2
|                                |ATTRIBUTE_CHAR_VALU                             |3
|                                |DESC order of PRIMARY_ENTITY_KEY(AKA PARTY_KEY) |4 
Sort by PRIMARY_ENTITY_KEY is added in case the same PARTY_NUMBER has different PARTY_KEY. This is to ensure that
when joining the multiple attribute score tables in the CRR process with have the most recent 
PARTY_KEY



PREP DATA SET(S) VOLUME
-----------------------
1. One to many ATTRIBUTE_CODE per PARTY_KEY


PREP DATA SET(S) NOTES
----------------------
1. Source data to support ATTRIBUTE_CODE and ATTRIBUTE_CHAR_VALUE:
   OCCUPATION: FSC_PARTY_DIM.OCCUPATION_DESC              
   INDUSTRY: FSC_PARTY_DIM.INDUSTRY_CODE
   PEP: FSC_PARTY_DIM.POLITICALLY_EXPOSED_PERSON_IND
   NON_PROFILE: NON_PROFIT_ORG_IND
   SAR_FILED: FSC_PARTY_DIM.SAR_FILED



================================================================================
RULE(S) SUPPORTED:


==============================================================================*/

/*-------------------------------------------------------------------
 * Copyright (c) 2013-2014 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/

%macro cdd_prep_entity_risk;

    /* Create entity attribute prep file from FSC_PARTY_DIM */
    proc contents data=seg_kc.fsc_party_dim out=party_meta NOPRINT;
    run;

    %let ex_party_calculated_column_list='SAR_COUNT';
    %let ex_party_column_list=;
    proc sql noprint;
             select "'"||strip(kupcase(name))||"'"
                             into :ex_party_column_list separated by ' '
             from party_meta
    ;
    quit;

    data entity_attribute_list;
         length party_type_desc   $ 20
                attribute_type    $ 3
                party_column_name $ 30
                attribute_code    $ 32

                ref_table_nm      $30
                lookup_level_2_type $28
         ;
         infile '!CDDROOT/custom/source/cdd_entity_attr_list.csv'
                 dsd lrecl=32764 truncover firstobs=2  delimiter=',';
         input party_type_desc
               attribute_type
               party_column_name
               attribute_code
               ref_table_nm
               lookup_level_2_type
         ;
         if party_column_name ne '' and attribute_code ne ''
            and party_column_name in (&ex_party_column_list &ex_party_calculated_column_list)
         then do;
            output;
            call symput('party_type_desc'||strip(put(_n_,best.))
                        , "'"||strip(party_type_desc)||"'"
                        );
            call symput('attribute_type'||strip(put(_n_,best.))
                        , "'"||strip(attribute_type)||"'"
                        );
            call symput('party_column_name'||strip(put(_n_,best.))
                        , strip(party_column_name)
                        );
            call symput('attribute_code'||strip(put(_n_,best.))
                        , "'"||strip(attribute_code)||"'"
                        );
            call symput('lookup_level_2_type'||strip(put(_n_,best.))
                        , "'"||strip(lookup_level_2_type)||"'"
                        );
            call symput('ref_table_nm'||strip(put(_n_,best.))
                        , "'"||strip(ref_table_nm)||"'"
                        );
            call symput('ENTITY_ATTR_N'
                        ,strip(put(_n_,best.))
                       );
         end;
    run;

   %put Calculating SAR count for forms: &sar_form_list;
    proc sql;
        create table party_sar_count as
            select pty.PARTY_NUMBER,
                   count(*) as SAR_COUNT
                from seg_kc.fsc_party_dim as pty
                     inner join seg_kc.fsk_rc_report_entity as ent
                        on  pty.party_number=ent.entity_number
                            and ent.entity_level_code='PTY'
                            and ent.LOGICAL_DELETE_IND ne 'Y'
                            and pty.change_current_ind='Y'
                     inner join seg_kc.fsk_rc_report as rpt
                        on  ent.REPORT_KEY = rpt.REPORT_KEY
                            and rpt.form_config_key in (&sar_form_list)
                            and rpt.report_ready_code in ('A', 'G')
                group by party_number;
    quit;
    %fcf_rcset(&syserr)
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
         %fcf_put_error;
         %goto MACRO_END;
    %end;

    %let party_column_list=;
    proc sql noprint;
         select distinct ', b.'||strip(party_column_name) 
                         into :party_column_list separated by ' '
             from entity_attribute_list e
                  inner join party_meta m on m.name = e.party_column_name
         ;


        %put Processing party columns: &party_column_list;

        create table entity_party_dim as
        select distinct
                a.primary_entity_number
               ,a.party_type_desc
               ,a.party_number
               ,s.SAR_COUNT
               &party_column_list
            from cjobstat.cddds_entity_list as a
                left join seg_kc.fsc_party_dim as b
                    on a.party_number=b.party_number
                       and b.change_current_ind='Y'
                left join  party_sar_count s
                     on a.party_number=s.party_number
            order by primary_entity_number
        ;
    quit;

    %fcf_rcset(&syserr)
    %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
         %fcf_put_error;
         %goto MACRO_END;
    %end;

    data cddds_entity;
         
         set entity_party_dim 
          ;
          
          length ATTRIBUTE_CODE $32
                 ATTRIBUTE_CHAR_VALUE $32
                 LOOKUP_SCORE 8
                 REF_TABLE_NM $30
          ;
          label ATTRIBUTE_CODE='ATTRIBUTE_CODE'
                ATTRIBUTE_CHAR_VALUE='ATTRIBUTE_CHAR_VALUE'
                LOOKUP_SCORE='LOOKUP_SCORE'
                REF_TABLE_NM='REF_TABLE_NM'
          ;   

          %do entity_attr_i=1 %to &ENTITY_ATTR_N;
            
              if party_type_desc = &&party_type_desc&entity_attr_i then
              do;
                  %if &&attribute_type&entity_attr_i = 'NUM' %then
                    %let cond = &&party_column_name&entity_attr_i ne .;
                  %else
                    %let cond = &&party_column_name&entity_attr_i ne '';

                  if &cond then
                  do;
                      lookup_score=.;
                      ref_table_nm=&&ref_table_nm&entity_attr_i;
                      attribute_code=&&attribute_code&entity_attr_i;
                      %if &&attribute_type&entity_attr_i = 'NUM' %then %do;
                           attribute_char_value=put(&&party_column_name&entity_attr_i, 8.);
                      %end;
                      %else %do;
                          attribute_char_value=&&party_column_name&entity_attr_i;
                          if attribute_char_value > '' then
                          do;
                              %if &&lookup_level_2_type&entity_attr_i ne ''  %then %do;

                                  lookup_score=input(putc(attribute_char_value, &&lookup_level_2_type&entity_attr_i)
                                                     ,best.
                                                     );
                              %end;
                          end;
                      %end;
                      output;
                  end;
              end;
          %end;

    run;

    %fcf_rcset(&syserr)
     %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
         %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
         %fcf_put_error;
         %goto MACRO_END;
     %end;  

/* Left join the prep file with FSC_PARTY_DIM */
/* to make sure that there is at least one record per internal party */
/* All Prep file must contain the following columns */
/*    PRIMARY_ENTITY_TYPE_CODE - 'PTY'for party, 'ACC' for account, 'HHD' for household data   */ 
/*    PRIMARY_ENTITY_KEY - depending on PRIMARY_ENTITY_TYPE_CODE, it should contain            */
/*                         PARTY_KEY, ACCOUNT_KEY, or HOUSEHOLD_KEY value                      */ 
/*    PRIMARY_ENTITY_NUMBER - depending on PRIMARY_ENTITY_TYPE_CODE, it should contain         */
/*                            PARTY_NUMBER, ACCOUNT_NUMBER, or HOUSEHOLD_NUMBER value          */ 

    proc sql ;
         create table cm_prep.cddds_entity_risk_prep as
         select   distinct
                  a.*
                , b.attribute_code 'Attribute Code'
                , b.attribute_char_value 'Attribute Value (char)'
                , b.lookup_score 'Lookup Score'
                , b.ref_table_nm
         from cjobstat.cddds_entity_list
                 /* remove party_number from output to main only one unique attribute result per entity */
                    (drop=party_number)  
                  as a
         left join  cddds_entity as b 
         on a.primary_entity_number=b.primary_entity_number
         order by primary_entity_number, attribute_code, attribute_char_value, primary_entity_key DESC
     
         ;
    quit;

    %fcf_rcset(&syserr)
             %if &trans_rc eq 3 or &trans_rc ge 5 %then %do;
                  %let _fcfmsg_=%sysfunc(sasmsg(sashelp.cddmsg,CDD_CRR_MACRO_ABORT,NOQUOTE,&sysmacroname))  ;
                  %fcf_put_error;
                  %goto MACRO_END;
            %end;  

 %MACRO_END:

   %if &trans_rc eq 3 or &trans_rc ge 5 %then
      %cdd_error;
   %else %do;
      %cdd_success;
   %end;

%mend;
