/* MAINDOC
+------------------------------------------------------------------------------+
| NAME:    rc_efile_wrt_entity_info.sas
| PRODUCT: SAS Regulatory Console
| SYSTEM:  Windows, Unix
| DESC:    This macro is for creating json strings for all the data in sas table.
|          The resulting data will be stored in FSK_RC_REPORT_ENTITY_INFO. 
|
+-----------------------------------------------------------------------------------------+
| USAGE:   indsn = input sas table
|          entity_type_name = string to be populated in FSK_RC_REPORT_ENTITY_INFO.ENTITY_TYPE_NAME
|          entity_level_code = string to be populated in FSK_RC_REPORT_ENTITY_INFO.ENTITY_LEVE_CODE
|          entity_key_var = name of the input field that will be used for  
|                  FSK_RC_REPORT_ENTITY_INFO.ENTITY_KEY
|          entity_number_var = name of the input field that will be used for  
|                  FSK_RC_REPORT_ENTITY_INFO.ENTITY_NUMBER
|          entity_name_var = name of the input field that will be used for  
|                  FSK_RC_REPORT_ENTITY_INFO.ENTITY_NAME
|          where_cl = condition to filter the input data
| NOTE: for date fields, the format must be DATETIME.
 +------------------------------------------------------------------------------------------+
MAINDOCEND */

/*-------------------------------------------------------------------
 * Copyright (c) 2016 by SAS Institute Inc., Cary, NC 27513 USA
 *  ---All Rights Reserved.
 *-------------------------------------------------------------------*/


%macro rc_efile_wrt_entity_info    (indsn=,
                                    entity_type_name=,
                                    entity_level_code=,
                                    entity_key_var=,
                                    entity_number_var=,
                                    entity_name_var=,
                                    entity_description_var=,
                                    entity_from_date_var=,
                                    entity_to_date_var=,
                                    where_cl=
                                    );

options missing=''; 
%let entity_type_name=%upcase(&entity_type_name);
%let entity_level_code=%upcase(&entity_level_code);


/* get the column names, type, and format of the input table */
proc contents data=&indsn out=col_list (keep=name type format length ) noprint;
run;
 
filename gencode temp lrecl=256;
data _null_;
    file gencode; 
    set col_list end=eof;
    where name not in ('CURRENT_IND' 'EFFECTIVE_FROM_DATE' 'EFFECTIVE_TO_DATE' "%upcase(&entity_key_var)");
    length stmt $ 1024;
    retain stmt;
    length camelCaseName JsonNodeName JsonNodeValue $ 256;
    camelCaseName=compress(propcase(name,'_'),'_');
    camelCaseName=lowCase(substr(camelCaseName,1,1))||substr(camelCaseName,2);
    if index(upcase(name),'INSTITUTION_SF_TYPE_CODE') and type=2 and length > 1 then link SF_ARRYCAT;
    else
    do;        
        JsonNodeName='"'||strip(camelCaseName)||'"'||':';
        if type=2 then
           JsonNodeValue='quote(strip('||strip(name)||'),''"'')';
        else 
        do;
           if upcase(format)='DATETIME' then
               JsonNodeValue='quote(ifc('||strip(name)||' > 0,'||'put(datepart('||strip(name)||'),yymmdd10.)||":00:00.000+00:00","") ,''"'')';
           else 
            JsonNodeValue='quote(strip(put('||strip(name)||',best.)),''"'')';
        end;
        link col_header;
    end;
   
    link col_trailer;
  
    ;
    return;


COL_HEADER:
 stmt='ui_json=strip(ui_json)||'
      ||"'"||strip(jsonNodeName)||"'"
      ||'||'
      ||strip(jsonNodeValue)
      ||";";
 put stmt;
 
return;

COL_TRAILER:
  if not eof then 
  do;
    stmt='ui_json=strip(ui_json)||",";';
    put stmt;
  end;
return;

SF_ARRYCAT:
stmt='if INSTITUTION_SF_TYPE_CODE > "" then ';
put stmt;
stmt='do icode=1 to length(INSTITUTION_SF_TYPE_CODE);';
put stmt;
stmt='   ui_json=strip(ui_json)||''"institutionSfTypeCode''||substrn(INSTITUTION_SF_TYPE_CODE,icode,1)||''":true'';';
put @4 stmt ;
stmt="   if icode < length(INSTITUTION_SF_TYPE_CODE) then ui_json=strip(ui_json)||',';";
put @4 stmt ;
stmt='end;';
put stmt;
stmt='else ';
put stmt;
stmt='   ui_json=strip(ui_json)||''"institutionSfTypeCodeA":false'';';
put @4 stmt ;



return;

run;
 

data rc_report_entity_info;
     set &indsn;
     %if %nrbquote(where_cl) ne %nrbquote() %then
     %do;
         where &where_cl;
     %end;
     if 0 then set seg_kc.fsk_rc_report_entity_info;
     entity_type_name="&entity_type_name";
     entity_level_code="&entity_level_code";
     entity_key=&entity_key_var;
     %if %nrbquote(&entity_number_var) ne %nrbquote()  %then
     %do; 
     entity_number=&entity_number_var;
     %end;

     %if %nrbquote(&entity_name_var) ne %nrbquote()  %then
     %do;
     entity_name=&entity_name_var;
     %end;

     %if %nrbquote(&entity_description_var) ne %nrbquote()  %then
     %do; 
     entity_description=&entity_description_var;
     %end;

     %if %nrbquote(&entity_from_date_var) ne %nrbquote()  %then
     %do; 
     effective_from_date=&entity_from_date_var;
     %end;

     %if %nrbquote(&entity_to_date_var) ne %nrbquote()  %then
     %do; 
     effective_to_date=&entity_to_date_var;
     %end;

     ui_json='{';
     %inc gencode;
     ui_json=strip(ui_json)||'}';
     output;
     keep entity_type_name entity_level_code entity_key entity_number entity_name entity_description 
          ui_json effective_from_date effective_to_date ;
run;


%if &rc_debug=N %then
%do;

proc sql;
     delete from seg_kc.fsk_rc_report_entity_info
     where entity_type_name="&entity_type_name";
quit;

proc append base=seg_kc.fsk_rc_report_entity_info (dbforce=yes)
            data=rc_report_entity_info force ;
run;

%end;

%mend;



